package no.ntnu.idatt1002.demo.controller;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.text.Text;
import javafx.stage.Modality;
import javafx.stage.Stage;
import no.ntnu.idatt1002.demo.data.Budget.BudgetItem;
import no.ntnu.idatt1002.demo.data.Budget.FileHandlingBudget;
import no.ntnu.idatt1002.demo.data.Budget.GeneralBudget;
import no.ntnu.idatt1002.demo.data.Economics.ExpenseCategory;


import java.io.IOException;
import java.util.Optional;

public class BudgetController {

    private GeneralBudget general;

    @FXML
    private Button addBudget;

    @FXML
    private Button editBudget;

    @FXML
    private Button expenseBtn;

    @FXML
    private Button incomeBtn;

    @FXML
    private Button returnBtn;

    @FXML
    private Button nextBtn;

    @FXML
    private TableColumn<BudgetItem, Double> amountColumn;

    @FXML
    private TableView<BudgetItem> budgetTableView = new TableView<>();

    @FXML
    private TableColumn<BudgetItem, ExpenseCategory> categoryColumn;

    @FXML
    private TableColumn<BudgetItem, String> descriptionColumn;

    @FXML
    private Text sum;

    @FXML
    private TableColumn<BudgetItem, Double> percentageColumn;

    @FXML
    private ObservableList<BudgetItem> budgetList;



    public void initialize() throws IOException {
        categoryColumn.setCellValueFactory(new PropertyValueFactory<BudgetItem, ExpenseCategory>("budgetCategory"));
        amountColumn.setCellValueFactory(new PropertyValueFactory<BudgetItem, Double>("budgetAmount"));
        descriptionColumn.setCellValueFactory(new PropertyValueFactory<BudgetItem, String>("budgetDescription"));

        general = loadIncomeDataFromFile("Budget");
        budgetList = FXCollections.observableArrayList(general.getBudgetItems());
        budgetTableView.setItems(budgetList);

        sum.setText(String.valueOf(general.totalSum()));
    }
    @FXML
    public void switchAddBudget(javafx.event.ActionEvent event) throws IOException {
        BudgetItem item = null;
        String dialogTitle = "";
        FXMLLoader loader = new FXMLLoader(SceneController.class.getResource("/view/AddBudget.fxml"));
        Dialog<BudgetItem> dialog = new Dialog<>();
        dialog.initModality(Modality.APPLICATION_MODAL);

        try{
            dialog.getDialogPane().setContent(loader.load());
        } catch(IOException e) {
            e.printStackTrace();
        }

        AddBudgetController budgetController = loader.getController();

        DialogMode dialogMode;
        if(event.getSource().equals(addBudget)){
            dialogMode = DialogMode.ADD;
            dialogTitle = "New Budget";

        }
        else if (event.getSource().equals(editBudget) && budgetTableView.getSelectionModel().getSelectedItem() != null) {
            dialogMode = DialogMode.EDIT;
            dialogTitle = "Edit expense";
            item = budgetTableView.getSelectionModel().getSelectedItem();
            budgetController.setBudget(item);
        } else {
            return;
        }

        dialog.setTitle(dialogTitle);
        dialog.showAndWait();

        item = budgetController.getNewBudgetItem();
        if(item != null && dialogMode == DialogMode.ADD){
            try {
            general.addToBudgetBudgetItem(item);
            } catch(IllegalArgumentException e) {
                showIllegalBudgetItemDialog();
            }
        }
        refreshTableView();
    }

    @FXML
    public void closeButton(javafx.event.ActionEvent actionEvent) {
        final Node source = (Node) actionEvent.getSource();
        final Stage stage = (Stage) source.getScene().getWindow();
        stage.close();
    }

    @FXML
    public void deleteButton(ActionEvent event) {
        BudgetItem item = budgetTableView.getSelectionModel().getSelectedItem();
        if (item == null) {
            return;
        }
        Optional<ButtonType> isConfirmed = showConfirmationDialog();
        if (isConfirmed.isPresent() && isConfirmed.get() == ButtonType.OK) {
            general.deleteItemFromBudget(item.getBudgetCategory());
            refreshTableView();
        }
    }

    private Optional<ButtonType> showConfirmationDialog() {
        Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
        alert.setTitle("Confirm Delete");
        alert.setHeaderText("Delete Confirmation");
        alert.setContentText("Are you sure you would like to delete the selected entry?");

        return alert.showAndWait();
    }

    private Optional<ButtonType> showIllegalBudgetItemDialog() {
        Alert alert = new Alert(AlertType.ERROR);
        alert.setTitle("Budget amount exceeded");
        alert.setHeaderText("Your budget exceeds the max limit");
        alert.setContentText("The total budget sum must be below " + general.getMaxAmount());

        return alert.showAndWait();
    }


    public GeneralBudget loadIncomeDataFromFile(String fileName) throws IOException {
        FileHandlingBudget fileHandlingBudget = new FileHandlingBudget();
        if (fileHandlingBudget.isEmpty(fileName)) {
            general = new GeneralBudget(31, 1000);
        } else {
            try {
                general = fileHandlingBudget.readGeneralBudgetFromFile(fileName);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return general;
    }

    public void saveDataToFile(String fileName) throws IOException {
        FileHandlingBudget fileHandlingBudget = new FileHandlingBudget();
        fileHandlingBudget.writeGeneralBudgetToFile(fileName, general);
    }


    protected void refreshTableView(){
        this.budgetList.setAll(general.getBudgetItems());
        this.sum.setText(String.valueOf(general.totalSum()));
    }

   @FXML
    public void switchScene(ActionEvent event) throws IOException {
        saveDataToFile("Budget");
        FXMLLoader loader = new FXMLLoader();
        if (event.getSource() == expenseBtn) {
            loader.setLocation(SceneController.class.getResource("/view/Expenses.fxml"));
        } else if (event.getSource() == returnBtn) {
            loader.setLocation(SceneController.class.getResource("/view/FirstMenu.fxml"));
        } else if (event.getSource() == incomeBtn) {
            loader.setLocation(SceneController.class.getResource("/view/Income.fxml"));
        } else if (event.getSource() == nextBtn) {
            loader.setLocation(SceneController.class.getResource("/view/MainMenu.fxml"));
        }
        Parent root = loader.load();
        Stage stage = (Stage) ((Node) event.getSource()).getScene().getWindow();
        Scene scene = new Scene(root);
        stage.setScene(scene);
        stage.show();
    }
}
