package no.ntnu.idatt1002.demo.controller;
import java.io.IOException;
import java.time.LocalDate;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.DatePicker;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.image.ImageView;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import no.ntnu.idatt1002.demo.data.Economics.ExpenseRegister;
import no.ntnu.idatt1002.demo.data.Economics.FileHandling;
import no.ntnu.idatt1002.demo.data.Economics.IncomeRegister;

public class MainMenuController {

    @FXML
    private Button addExpenseBtn;

    @FXML
    private Button foodButton;

    @FXML
    private Button overviewBtn;

    @FXML
    private ProgressBar progressbar;

    @FXML
    private DatePicker date;

    @FXML
    private ImageView progressMarker;

    @FXML
    private Label today;

    @FXML
    private Text budgetMonth;

    @FXML
    private Label balanceLbl;


    @FXML
    public void initialize() throws IOException {
        IncomeController incomeController = new IncomeController();
        IncomeRegister incomeRegister = incomeController.loadIncomeDataFromFile("Income");

        ExpensesController expensesController = new ExpensesController();
        ExpenseRegister expenseRegister = expensesController.loadExpenseDataFromFile("Expense");

        double incomeSum = incomeRegister.getTotalSum();
        double expenseSum = expenseRegister.getTotalSum();

        progressbar.setProgress(expenseSum/incomeSum);
        progressMarker.setTranslateX(-275 + progressbar.getProgress());

        budgetMonth.setText("BUDGET " + (LocalDate.EPOCH.getMonth()));
        double balance = incomeSum - expenseSum;
        balanceLbl.setText("Balance: " + (balance));

        today.setText("Used " + expenseSum + " out of " + incomeSum + " this month");

        if (balance < 0) {
            balanceLbl.setTextFill(Color.RED);
        }

        date.setValue(LocalDate.now());
        //date.restrict
    }

    @FXML
    public void switchScene(ActionEvent event) throws IOException {
        //saveDataToFile("Income");
        FXMLLoader loader = new FXMLLoader();
        if (event.getSource() == addExpenseBtn) {
            loader.setLocation(SceneController.class.getResource("/view/Expenses.fxml"));
        } else if (event.getSource() == overviewBtn) {
            loader.setLocation(SceneController.class.getResource("/view/Budget.fxml"));
        }

        Parent root = loader.load();
        Stage stage = (Stage) ((Node) event.getSource()).getScene().getWindow();
        Scene scene = new Scene(root);
        stage.setScene(scene);
        stage.show();
    }
}
